<?php

namespace Quivers\Setup;

use Doctrine\DBAL\Connection;
use Shopware\Bundle\AttributeBundle\Service\CrudService;
use Shopware\Components\Model\ModelManager;
use Shopware\Models\Payment\Payment;
use Shopware\Models\Plugin\Plugin;
use Shopware\Components\Plugin\PaymentInstaller;
use Shopware\Components\Plugin\Context\InstallContext;
use Quivers\Bootstrap\Database;

class Installer
{
    /**
     * @var ModelManager
     */
    private $modelManager;

    /**
     * @var Connection
     */
    private $connection;

    /**
     * @var CrudService
     */
    private $attributeCrudService;

    /**
     * @var string
     */
    private $bootstrapPath;

    /**
     * @var PaymentInstaller
     */
    private $paymentInstaller;

    /**
     * @var InstallContext
     */
    private $context;


    public function __construct(
        ModelManager $modelManager,
        Connection $connection,
        CrudService $attributeCrudService,
        PaymentInstaller $paymentInstaller,
        InstallContext $context
    ) {
        $this->modelManager = $modelManager;
        $this->connection = $connection;
        $this->attributeCrudService = $attributeCrudService;
        $this->paymentInstaller = $paymentInstaller;
        $this->context = $context;
    }

    /**
     * @throws InstallationException
     *
     * @return bool
     */
    public function install()
    {
        $this->installStripe();
        $this->createDatabaseTables();
        $this->createAttributes();
        $this->addReturndedStatus();
        return true;
    }

   
    private function createDatabaseTables()
    {
        $database = new Database($this->modelManager);
        $database->install();
    }

    private function installStripe(){
        $options = [
            'name' => 'Quivers Stripe',
            'description' => 'Pay By Card',
            'action' => 'QuiversStripePayment',#controller responsible for this action
            'class' => 'QuiversStripePaymentCard',
            'active' => 0,
            'position' => 1,
            'additionalDescription' => ''
        ];
        $this->paymentInstaller->createOrUpdate($this->context->getPlugin(), $options);

    }


    private function createAttributes()
    {
        $this->attributeCrudService->update('s_order_details_attributes', "trackingCode_1", 'string', [
            'custom' => true,
            'supportText' => 'Quivers Shipment 1',
            'label' => "Tracking Code",
            'displayInBackend' => true,
        ], null, true);
       $this->attributeCrudService->update('s_order_details_attributes', "quantityShipped_1", 'integer', [
            'custom' => true,
            'supportText' => 'Quivers Shipment 1',
            'label' => "Quantity Shipped",
            'displayInBackend' => true,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "shippingCarrier_1", 'string', [
            'custom' => true,
            'supportText' => 'Quivers Shipment 1',
            'label' => "Shipping Carrier",
            'displayInBackend' => true,
        ], null, true);
        $this->attributeCrudService->update('s_order_details_attributes', "taxRefunded_1", 'float', [
            'custom' => true,
            'supportText' => 'Quivers Shipment 1',
            'label' => "Tax Refunded",
            'displayInBackend' => true,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "shippingFee_1", 'float', [
            'custom' => true,
            'supportText' => 'Quivers Shipment 1',
            'label' => "Shipping Fee Refunded",
            'displayInBackend' => true,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "quantity_returned", 'integer', [
            'custom' => true,
            'label' => "(Quivers) Quantity Returned",
            'displayInBackend' => true,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "amount_refunded", 'float', [
            'custom' => true,
            'label' => "(Quivers) Amount Refunded",
            'displayInBackend' => true,
        ], null, true, 0);
        $this->attributeCrudService->update('s_order_details_attributes', "cancellation_quantity", 'integer', [
            'custom' => true,
            'label' => "(Quivers) Quantity Cancelled",
            'displayInBackend' => true,
        ], null, true, 0);
        $this->attributeCrudService->update('s_articles_attributes', "product_tax_code", 'string', [
            'custom' => true,
            'label' => "Product Tax Code",
            'helpText' => 'Quivers Product Tax Code',
            'displayInBackend' => true,
        ]);
        $this->attributeCrudService->update('s_articles_attributes', "quivers_hidden_upc", 'string', [
            'custom' => true,
            'label' => "UPC hidden value",
            'helpText' => '',
            'displayInBackend' => false,
        ]);
        $this->attributeCrudService->update('s_articles_attributes', "quivers_upc", 'string', [
             'custom' => true,
             'label' => "UPC",
             'helpText' => 'Quivers UPC field',
             'displayInBackend' => true,
        ]);
        $this->attributeCrudService->update('s_articles_attributes', "quivers_stock", 'integer', [
             'custom' => true,
             'label' => "Stock on Quivers",
             'helpText' => 'Quivers Stock field',
             'displayInBackend' => true,
        ]);
        $this->attributeCrudService->update('s_order_attributes', "changed", 'datetime', [
            'custom' => true,
            'label' => "changed",
            'displayInBackend' => false,
        ], null, true);
        $this->attributeCrudService->update('s_articles_attributes', "last_modified_at", 'datetime', [
            'custom' => true,
            'label' => "Modified At",
            'displayInBackend' => false,
        ]);
        $this->modelManager->generateAttributeModels(['s_order_details_attributes', 's_articles_attributes',
        's_order_attributes']);
    }

    public function addReturndedStatus (){
        $sql = $this->connection->createQueryBuilder()
                ->select('*')
                ->from('s_core_detail_states')
                ->where('description = :description')
                ->setParameter('description', 'Returned')
                ->execute()->fetchAll();
        if (!$sql){
            $data = [
                "id" => 4,
                "description" => "Returned",
                "position" => 5,
                "mail" => 0
            ];
            $this->connection->insert("s_core_detail_states", $data);
        }
        $sql = $this->connection->createQueryBuilder()
                ->select('*')
                ->from('s_core_states')
                ->where('name = :name')
                ->setParameter('name', 'Returned')
                ->execute()->fetchAll();
        if (!$sql){
            $data = [
                "id" => 36,
                "name" => "Returned",
                "description" => "Return Order",
                "position" => 10,
                "`group`" => "state",
                "mail" => 0
            ];
            $this->connection->insert("s_core_states", $data);
        }
    }

}


